package SPVM::Callback;

1;

=head1 Name

SPVM::Callback - Callback interface

=head1 Usage
  
  use Callback;

=head1 Description

The Callback interface is an L<interface|SPVM::Document::Language/"Interface"> for callbacks.

=head1 Interface Methods

=head2 Anon Method

C<required method : void ();>

A callback.

=head1 Copyright & License

Copyright (c) 2023 Yuki Kimoto

MIT License
