package SPVM::Error::NotSupported;

1;

=head1 Name

SPVM::Error::NotSupported - Not Supported Error

=head1 Usage

  use Error::NotSupported;

=head1 Description

The Error::NotSupported class represents errors the system dose not support this feature.

=head1 Super Class

L<Error|SPVM::Error>

=head1 Copyright & License

Copyright (c) 2023 Yuki Kimoto

MIT License
