// Copyright (c) 2023 Yuki Kimoto
// MIT License

#ifndef SPVM_PUBLIC_API_H
#define SPVM_PUBLIC_API_H

#include "spvm_typedecl.h"
#include "spvm_native.h"

SPVM_ENV* SPVM_PUBLIC_API_new_env();

#endif
